#-----------------------------------------------------------------------------
# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Group.pm,v 1.12 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.12 $
#
# Provides: package Group
#
# Functionality: VCS Group Related Commands
#
# Authors: 011787
#-----------------------------------------------------------------------------

#---------------
# version calls.
#---------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Group.pm,v 1.12 2009/07/27 11:59:51 aatre Exp $::ETVHeader");
#-----------------------
# end of version calls.
#-----------------------

#----------------------------
# Declare the package name.
#----------------------------
package Group;

#----------------------------------------
# turn on strict on subs, refs and vars.
#----------------------------------------
use strict;

#-----------------------------------
# Include all the modules required.
#-----------------------------------
use LogInt;
use Hagrp;
use Hares;
use Resource;

#-----------------------------------------------
# internal const defines (for OnlineTypeFlag).
#-----------------------------------------------
use constant VCSDEF_GROUP_ONLINE_WITH_PREONLINE    => 1;
use constant VCSDEF_GROUP_ONLINE_WITHOUT_PREONLINE => 2;


=pod

-----------------------------------------------------------------------------
Function: Boolean Online( IN String GroupName, 
                          IN String SystemName,
                          IN Integer WaitPeriod )

Description:
  Online service group on specified system.
 
Parameters:
  GroupName  : Name of the service group to be online.
  SystemName : Name of the host on which the service group needs to bring online.
  WaitPeriod : Number of seconds to wait before the service group comes online.

Return Value:
  Boolean value from the set of {0,1} where
         0 - The group failed to come online
         1 - The group came online.
-----------------------------------------------------------------------------

=cut

sub Online {
 
  my ( $sGroupName,
       $sSystemName, 
       $iWaitPeriod ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10370,
                   "Group",
                   "Online",
                   "Subroutine %s called with: Group Name [%s], System Name [%s], Wait Period [%s]",
                   $sMyName, $sGroupName, $sSystemName, $iWaitPeriod );

  LogInt::Message( LogInt::LLTrace,
                   10371,
                   "Group",
                   "Online",
                   "Group[$sGroupName] System[$sSystemName] WaitPeriod[$iWaitPeriod] calling OnlineInternal" );

  #----------------------------------------------------------
  # call the internal call with the 'preonline' flag on.
  #----------------------------------------------------------
  return OnlineInternal( $sGroupName,
                         $sSystemName,
                         VCSDEF_GROUP_ONLINE_WITH_PREONLINE,
                         $iWaitPeriod );
}


=pod

-----------------------------------------------------------------------------
Function: OnlineNoPreonline( IN String GroupName, 
                             IN String SystemName, 
                             IN Integer WaitPeriod )

Description:
  Online service group without checking for per-online scripts.

Parameters:
  GroupName  : Name of the service group to be online.
  SystemName : Name of the host on which the service group needs to bring online.
  WaitPeriod : Number of seconds to wait before the service group comes online.

Return Value:
  Boolean value from the set of {0,1} where
          0 - The group failed to come online.
          1 - The group came online.
-----------------------------------------------------------------------------

=cut

sub OnlineNoPreonline {

  my ( $sGroupName,
       $sSystemName,
       $iWaitPeriod ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10372,
                   "Group",
                   "OnlineNoPreonline",
                   "Subroutine %s called with: Group Name [%s], System Name [%s], Wait Period [%s]",
                   $sMyName, $sGroupName, $sSystemName, $iWaitPeriod );

  LogInt::Message( LogInt::LLTrace,
                   10373,
                   "Group",
                   "OnlineNoPreonline",
                   "Group[$sGroupName] System[$sSystemName] WaitPeriod[$iWaitPeriod] calling OnlineInternal");

  #----------------------------------------------------------
  # call the internal call with the 'preonline' flag off.
  #----------------------------------------------------------
  return OnlineInternal( $sGroupName,
                         $sSystemName,
                         VCSDEF_GROUP_ONLINE_WITHOUT_PREONLINE,
                         $iWaitPeriod );
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetIPsForServiceGroup( IN String ServiceGroup,
                                         OUT ListOfStrings IPList )

Description: 
  Gets all the IP addresses used by the given service group.

Parameters: 
  ServiceGroup : Name of the service group for which IP addresses are required.
  IPList       : The List of IP addresses used by the given service group.

Return Value:
  Boolean value from the set of {0,1} where
         0 - failure.
         1 - success.
-----------------------------------------------------------------------------

=cut

sub GetIPsForServiceGroup {

  my ( $sServiceGroup,
       $rlsIPList ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10374, 
                   "Group",
                   "GetIPsForServiceGroup", 
                   "Subroutine %s called with: Group Name [%s]",
                   $sMyName, $sServiceGroup );

  LogInt::Message( LogInt::LLTrace,
                   10375, 
                   "Group",
                   "GetIPsForServiceGroup", 
                   "service group:[$sServiceGroup], calling Hares::Display");

  #--------------------------------------------------------------
  # construct the call to the generic Hares Display command.
  #--------------------------------------------------------------
  my $sResources  = undef;
  my $sAttributes = "Address";
  my $sGroups     = $sServiceGroup;
  my $sTypes      = "IP";
  my $sSystems    = undef;
  my @lsResults   = ();

  if(Hares::Display($sResources, $sAttributes, $sGroups, $sTypes, $sSystems, \@lsResults)) {
    # return ok - continue
  } 
  else {
    #----------------------------------
    # call to get display info failed.
    #----------------------------------
    LogInt::Message( LogInt::LLError,
                     10376,
                     "Group",
                     "GetIPsForServiceGroup",
                     "failed in call to Hares::Display");
    return 0; #return error
  }

  for my $sEachResult (@lsResults) {
    my ( $sResourceTrash, 
         $sAttributeTrash,
         $sSystemTrash,
         $sValue ) = split(/\s/, $sEachResult);

    LogInt::Message( LogInt::LLTraceTrace,
                     10377,
                     "Group",
                     "GetIPsForServiceGroup",
                     "adding IP:[$sValue]" );
    push(@$rlsIPList, $sValue);
  }

  return 1;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean Offline( IN String GroupName,
                           IN String SystemName,
                           IN Integer WaitPeriod )

Description:
  Brings the given service group to offline.

Parameters:
  GroupName  : Name of the service group to be offline.
  SystemName : Name of the host on which the service
               group needs to bring offline.
  WaitPeriod : Number of seconds to wait before the
               service group becomes offline.

Return Value:
  Boolean value from the set of {0,1} where
          0 - The group failed to become offline
          1 - The group becomes offline.

-----------------------------------------------------------------------------

=cut

sub Offline {

  my ( $sGroupName,
       $sSystemName,
       $iWaitPeriod ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10378,
                   "Group", 
                   "Offline",
                   "Subroutine %s called with: Group Name [%s], System Name [%s], Wait Period [%s]",
                   $sMyName, $sGroupName, $sSystemName, $iWaitPeriod );

  #-----------------------------------------------------
  # if iWaitPeriod is zero wait forever
  # if iWaitPeriod is < zero output error and exit 0
  #-----------------------------------------------------

  if($iWaitPeriod < 0) {
    LogInt::Message( LogInt::LLError,
                     10379,
                     "Group", 
                     "Offline",
                     "Called with WaitPeriod[$iWaitPeriod]" );
    return 0; #error return
  }

  LogInt::Message( LogInt::LLTrace, 
                   10380,
                   "Group",
                   "Offline",
                   "Group[$sGroupName] System[$sSystemName] WaitPeriod[$iWaitPeriod]");

  #-------------------------
  # call offline function.
  #-------------------------
  if(Hagrp::Offline($sGroupName, $sSystemName)) {
    #command ran ok - continue
  } 
  else {
    #command failed - return error
    LogInt::Message( LogInt::LLError,
                     10381,
                     "Group",
                     "Offline",
                     "failed in call to Hagrp::Offline" );
    return 0;
  }

  #------------------------------------------------------
  # need to wait for the system to go offline
  # get state, waiting for the output to be 'OFFLINE'
  # or the output to contain 'FAULTED'
  # how long to wait depends on iWaitPeriod
  #------------------------------------------------------
  my $iTimeLeftToCheck = $iWaitPeriod;
  my $iCheckPeriod     = 10; #period to wait between checks
  my $iWaitForever     = 0;

  if($iWaitPeriod == 0) {
    $iWaitForever = 1;
  }

  while($iWaitForever || ($iTimeLeftToCheck > 0)) {
    my $sState = undef;
    if(Hagrp::GetStateGivenGroupAndSystem($sGroupName, $sSystemName, \$sState)) {
      #command ran ok - continue
    } 
    else {
      #-------------------------------
      # command failed - return error.
      #-------------------------------
      LogInt::Message( LogInt::LLError,
                       10382,
                       "Group",
                       "Offline",
                       "failed in call to Hagrp::GetStateGivenGroupAndSystem");
      return 0;
    }

    if($sState =~ /^OFFLINE$/) {
      #--------------------------------
      # set state to good, jump loop.
      #--------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10383,
                       "Group",
                       "Offline",
                       "Group[$sGroupName] System[$sSystemName] is OFFLINE");
      return 1; #group is offline - return good
    } 
    elsif($sState =~ /FAULTED/) {
      #-------------------------------
      # set state to fail, jump loop.
      #-------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10384,
                       "Group",
                       "Offline",
                       "Group[$sGroupName] System[$sSystemName] is FAULTED");
      return 0; #group is faulted - return error
    }

    LogInt::Message( LogInt::LLTraceTrace,
                     10385,
                     "Group",
                     "Offline",
                     "Group[$sGroupName] System[$sSystemName] waiting for offline");

    sleep $iCheckPeriod;
    $iTimeLeftToCheck = $iTimeLeftToCheck - $iCheckPeriod;
  }

  #-------------------------------------------
  #group didnt change state - return error.
  #-------------------------------------------

  return 0; 
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetGroupsCurrentSystem( IN String GroupName, 
                                          OUT String SystemName )

Description:
  Get the system name on which the service group is online.

Parameters:
  GroupName  : Name of the Service group.
  SystemName : Name of the system on which the group is online.

Return Value:
  Boolean value from the set of {0,1} where
          0 - failure.
          1 - success.
-----------------------------------------------------------------------------

=cut

sub GetGroupsCurrentSystem {

  my ( $sGroupName,
       $rsSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10386,
                   "Group",
                   "GetGroupsCurrentSystem",
                   "Subroutine %s called with: Group Name [%s]",
                   $sMyName, $sGroupName );

  LogInt::Message( LogInt::LLTrace,
                   10387,
                   "Group",
                   "GetGroupsCurrentSystem",
                   "Group:[$sGroupName], calling Hagrp::GetStateInfoListGivenGroup");

  #--------------------------------------------------------------------
  # get list of systems where this group exists
  # return the one which isnt showing it as offline
  # if group is showing offline on all systems, leave $rsSystem unset 
  #--------------------------------------------------------------------
  my @lsStateInfo = ();
  if(Hagrp::GetStateInfoListGivenGroup($sGroupName, \@lsStateInfo))
  {
    # all ok - continue
  } 
  else {
    #call to get display info failed
    LogInt::Message( LogInt::LLError,
                     10388,
                     "Group",
                     "GetGroupsCurrentSystem",
                     "failed in call to Hagrp::GetStateInfoListGivenGroup" );
    return 0; #return error
  }

  #-----------------------------------
  # output format is
  # Group Attribute System Value
  #-----------------------------------
  for my $sEachStateInfo (@lsStateInfo) {
    my ( $sGroupTrash, 
         $sAttributeTrash, 
         $sSystem, 
         $sState ) = split(/\s/, $sEachStateInfo);

    if($sState =~ /OFFLINE/) {
      LogInt::Message( LogInt::LLTraceTrace,
                       10389,
                       "Group",
                       "GetGroupsCurrentSystem",
                       "Ignoring Group:[$sGroupTrash], System:[$sSystem], State:[$sState]");
    } 
    else {
      LogInt::Message( LogInt::LLTrace,
                       10390,
                       "Group",
                       "GetGroupsCurrentSystem",
                       "Found Group:[$sGroupTrash], System:[$sSystem], State:[$sState], exiting...");
      $$rsSystemName = $sSystem;
      last; #jump loop
    }
  }

  return 1; #return ok
}


=pod

-----------------------------------------------------------------------------
Funcrion: Boolean OnlineInternal( IN String GroupName, 
                                  IN String  SystemName, 
                                  IN Constant OnlineType,
                                  IN Integer WaitPeriod )

Description: 
  It is called by Online subroutine and internally checks whether the
  group came online for specified wait period.

Parameters: 
  GroupName  : Name of the service group to be online.
  SystemName : Name of the host on which the service group needs to bring online.
  OnlineType : A constant to specifies one of the following values.
                 VCSDEF_GROUP_ONLINE_WITH_PREONLINE -
                   Indicates that HAD should not online a service group in 
                   response to an hagrp -online command or a fault. It should 
                   instead call a user-defined script that checks
                   for external conditions before bringing the group online.

                 VCSDEF_GROUP_ONLINE_WITHOUT_PREONLINE - 
                   Indicates HAD that no need to call user-defined scripts.
  WaitPeriod : Number of seconds to wait before the service group comes online.

Return Value:
  Boolean value from the set of {0,1} where
         0 - The group failed to come online.
         1 - The group came online.
-----------------------------------------------------------------------------

=cut

sub OnlineInternal {

  my ( $sGroupName,
       $sSystemName, 
       $bOnlineType,
       $iWaitPeriod ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10391,
                   "Group",
                   "OnlineInternal",
                   "Subroutine %s called with: Group Name [%s], System Name [%s], Online Type [%s], Wait Period [%s]",
                   $sMyName, $sGroupName, $sSystemName, $bOnlineType, $iWaitPeriod );

  #----------------------------------------------------
  # if iWaitPeriod is zero wait forever
  # if iWaitPeriod is < zero output error and exit 0
  #----------------------------------------------------

  if($iWaitPeriod < 0) {
    LogInt::Message( LogInt::LLError,
                     10392,
                     "Group",
                     "OnlineInternal",
                     "Called with WaitPeriod[$iWaitPeriod]");
    return 0; #error return
  }

  LogInt::Message( LogInt::LLTrace,
                   10393,
                   "Group",
                   "OnlineInternal",
                   "Group[$sGroupName] System[$sSystemName] WaitPeriod[$iWaitPeriod]");

  #-----------------------------
  # decide which to call here.
  #-----------------------------
  if($bOnlineType == VCSDEF_GROUP_ONLINE_WITH_PREONLINE) {
    if(Hagrp::Online($sGroupName, $sSystemName)) {
      #continue
    } 
    else {
      LogInt::Message( LogInt::LLError, 
                       10394,
                       "Group",
                       "OnlineInternal",
                       "Failed in call to Hagrp::Online" );
      return 0; #error return
    }

  } 
  elsif ($bOnlineType == VCSDEF_GROUP_ONLINE_WITHOUT_PREONLINE) {
    if(Hagrp::OnlineNoPre($sGroupName, $sSystemName)) {
      #continue
    }
    else {
      LogInt::Message( LogInt::LLError,
                       10395,
                       "Group",
                       "OnlineInternal",
                       "Failed in call to Hagrp::OnlineNoPre");
      return 0; #error return
    }
  } 
  else {
    LogInt::Message( LogInt::LLError, 
                     10396,
                     "Group",
                     "OnlineInternal",
                     "Incorrect value for OnlineType value[$bOnlineType]" );
    return 0; #error return
  }

  #----------------------------------------------------------------------
  # command ran ok, need to wait for the system to 
  # come online - how long depends on iWaitPeriod
  # get state waiting for the output to be 'ONLINE'
  # or the output to contain 'FAULTED' or for the WaitPeriod to expire
  #----------------------------------------------------------------------
  my $iTimeLeftToCheck = $iWaitPeriod;
  my $iCheckPeriod     = 10; #period to wait between checks
  my $iWaitForever     = 0;

  if($iWaitPeriod == 0) {
    $iWaitForever = 1;
  }

  while($iWaitForever || ($iTimeLeftToCheck > 0)) {
    my $sState = undef;
    if(Hagrp::GetStateGivenGroupAndSystem($sGroupName, $sSystemName, \$sState)) {
      #command ran ok - continue
    }
    else {
      #----------------------------------
      # command failed - return error.
      #----------------------------------
      LogInt::Message( LogInt::LLError,
                       10397,
                       "Group",
                       "OnlineInternal",
                       "failed in call to Hagrp::GetStateGivenGroupAndSystem" );
      return 0;
    }

    if($sState =~ /^ONLINE$/) {
      #set state to good - return
      LogInt::Message( LogInt::LLTrace,
                       10398,
                       "Group",
                       "OnlineInternal",
                       "Group[$sGroupName] System[$sSystemName] is ONLINE" );
      return 1; #group is online - return good
    }
    elsif($sState =~ /FAULTED/) {
      #------------------------------
      # set state to fail - return.
      #------------------------------
      LogInt::Message( LogInt::LLTrace, 
                       10399,
                       "Group",
                       "OnlineInternal",
                       "Group[$sGroupName] System[$sSystemName] is FAULTED" );
      return 0; #group is failed - return fail
    }

    LogInt::Message( LogInt::LLTrace,
                     10400,
                     "Group",
                     "OnlineInternal",
                     "Group[$sGroupName] System[$sSystemName] waiting for online");

    sleep $iCheckPeriod;
    $iTimeLeftToCheck = $iTimeLeftToCheck - $iCheckPeriod;
  }

  #-------------------------------------------------
  # group did not change state - return error
  #-------------------------------------------------
  return 0; 
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetSystemListHash( IN String GroupName,
                                     OUT Hash PrioritySystemNameHash )

Description:
  Gets all the hostname and their priorities in the cluster 
  on which the service group exists.

Parameters:
  GroupName              : Name of the service group.
  PrioritySystemNameHash : Hash with system name and it's priority.

Return Value:
  Boolean value in the set of {0,1} where
         1 - Returns on success.
         0 - Returns on failure.
-----------------------------------------------------------------------------

=cut

sub GetSystemListHash {

  my ( $sGroupName,
       $rhPrioritySystemNameHash ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10401,
                   "Group",
                   "GetSystemListHash",
                   "Subroutine %s called with Group Name [%s]",
                   $sMyName, $sGroupName );

  LogInt::Message( LogInt::LLTrace,
                   10402,
                   "Group",
                   "GetSystemListHash",
                   "service GroupName:[$sGroupName], calling Hagrp::Display" );

  #----------------------------------------------------------------
  # construct the call to the generic Hagrp Display command.
  #----------------------------------------------------------------

  my $sGroups     = $sGroupName;
  my $sAttributes = "SystemList";
  my $sSystems    = undef;
  my @lsResults   = ();

  if(Hagrp::Display($sGroups, $sAttributes, $sSystems, \@lsResults)) {
    # return ok - continue
  } 
  else {
    #------------------------------------
    # call to get display info failed.
    #------------------------------------
    LogInt::Message( LogInt::LLError,
                     10403,
                     "Group",
                     "GetSystemListHash",
                     "failed in call to Hagrp::Display" );
    return 0; #return error
  }

  #-----------------------------------------------------------------
  # format
  #   #Group         Attribute      System     Value
  #   VCS_ENGINE_HB  SystemList     global     helium 1 hydrogen 2
  #
  # convert the results entry into one string
  # (only the first entry has data in it)
  #-----------------------------------------------------------------
  my $sResultEntryString = $lsResults[0];

  #-------------------------------------------------
  # make the string into a list (split on spaces).
  #-------------------------------------------------
  my @lsSystemListData = split(/\s/, $sResultEntryString);

  #----------------------------------------------------------------
  # loose the first 3 params (they don't contain anything useful)
  #----------------------------------------------------------------
  shift @lsSystemListData;
  shift @lsSystemListData;
  shift @lsSystemListData;

  my $loop = 1;
  my %hTmpPrioritySystemNameHash = ();

  while ($loop) {
    my $sTmpSystemName = shift @lsSystemListData;

    if(defined $sTmpSystemName) {
      my $sTmpPriority = shift @lsSystemListData;
      if(defined $sTmpPriority) {
        $hTmpPrioritySystemNameHash{$sTmpPriority} = $sTmpSystemName;
        LogInt::Message( LogInt::LLTraceTrace,
                         10404,
                         "Group",
                         "GetSystemListHash",
                         "Adding SystemName [$sTmpSystemName] at Priority [$sTmpPriority]" );
      }
      else {
        LogInt::Message( LogInt::LLError,
                         10405,
                         "Group",
                         "GetSystemListHash",
                         "System list data invalid format [$sResultEntryString]" );
        return 0; #return error
      }
    } 
    else {
      $loop = 0;
    }
  }

  #-----------------------------------------------------------
  # assign the tmp hash contents to the passed in referance.
  #-----------------------------------------------------------
  %$rhPrioritySystemNameHash = %hTmpPrioritySystemNameHash;

  return 1;
}


=pod

-----------------------------------------------------------------------------
Function: String getGroupName( IN String ResourceName )

Description:
  Gives the group name for the given resource.

Parameters:
  ResourceName : Name of the resource.

Return Value:
  String : String containing the name of the group that 
           the given resource belongs.
-----------------------------------------------------------------------------

=cut

sub getGroupName {

  my ( $sResourceName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10406,
                   "Group",
                   "getGroupName",
                   "Subroutine %s called with: Group Name [%s]",
                   $sMyName, $sResourceName );

  my $sGroupName = undef;
  if(Resource::GetGroupNameGivenResource($sResourceName, \$sGroupName)) {
    return $sGroupName;
  }
  else {
    LogInt::Message( LogInt::LLTrace,
                     10407,
                     "Group",
                     "getGroupName",
                     "Call to getGroupName with ResourceName [%s] failed",
                     $sResourceName );
    return "";
  }
}


=pod

-----------------------------------------------------------------------------
Function: Boolean hagrp( IN String HaCmd, 
                         IN String GroupName,
                         IN String SystemName, 
                         IN Integer WaitPeriod )

Description:
  A rapper to the hagrp command internally calling corresponding subroutines.

Parameters:
  HaCmd      : Command which describes the one of the following actions:
                Online
                Offline
                OnlineNoPre
  GroupName  : Name of the service group in the cluster.
  SystemName : Name of the node/system in the cluster on which the group
               needs to bring Onbline, offline or onlineNoPre.
  WaitPeriod : Number of seconds to wait before the specified action completes.

Return Value:
  Boolean value in the set of {0, 1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub hagrp {

  my ( $sHaCmd,
       $sGroupName,
       $sSystemName,
       $iWaitPeriod ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10408,
                   "Group",
                   "hagrp",
                   "Subroutine %s called with: HA Command [%s], Group Name [%s], System Name [%s], Wait Period [%s]",
                   $sMyName, $sHaCmd, $sGroupName, $sSystemName, $iWaitPeriod );

  if ($sHaCmd eq "online") {
    return Group::Online($sGroupName, $sSystemName, $iWaitPeriod);
  }
  elsif ($sHaCmd eq "onlineNoPre") {
    return Group::OnlineNoPreonline($sGroupName, $sSystemName, $iWaitPeriod);
  }
  elsif ($sHaCmd eq "offline") {
    return Group::Offline($sGroupName, $sSystemName, $iWaitPeriod);
  }
  else {
    #------------------------------------
    # Unrecognised command argument.
    #------------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10409,
                     "Group",
                     "hagrp",
                     "Received invalid HaCmd string [%s]", $sHaCmd );
    return 0;
  }
}


1; #so perl knows its included ok

