# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/VCSHighLevel.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.16 $
#
# Provides: package VCSHighLevel
#
# Functionality: wrapper to high level VCS related calls 
#
# Authors: 011787
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
#   ProcType (one of Proc::PROC_TYPE_PARENT - match only parent process's 
#   Proc::PROC_TYPE_CHILDREN - match only children process's 
#   Proc::PROC_TYPE_ALL - match any (ie parent or child) process's 
#
#   PidFile full path of file to contain matched process Ids (or application process Ids)
#
#   CheckProcType - dictate what type of CheckProc to run 
#   types:
#   VCSHighLevel::CHECK_TYPE_APPLICATION - use application maintained pid file
#   ProcFilterList should be undefined (or empty)
#   NumberOfEachProcRequired should be undefined or < 2
#   ProcType - should be set to Proc::PROC_TYPE_ALL
#   PidFile - full path name of application pid file
#
#   VCSHighLevel::CHECK_TYPE_MAINTAINED_PIDFILE - script will generate/maintain pid file
#   ProcFilterList needs to contain at least one entry to match on
#   NumberOfEachProcRequired required number of matchs to be found
#   ProcType - any of the three, will determine what will be matched on
#   PidFile - full path name of pid file to use
#
#   VCSHighLevel::CHECK_TYPE_USE_PROCESSLIST - script will rely on 'ps' command
#   ProcFilterList needs to contain at least one entry to match on
#   NumberOfEachProcRequired required number of matchs to be found
#   ProcType - any of the three, will determine what will be matched on
#   PidFile - should be undefined
#-----------------------------------------------------------------------------

#-------------------
# version calls.
#-------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/VCSHighLevel.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $::ETVHeader");
#----------------------
# end of version calls
#----------------------

#---------------------------
# Declare the package name.
#---------------------------
package VCSHighLevel;

#--------------------------------------------
# Turn on strict for subs, refs and vars.
#--------------------------------------------
use strict;

#----------------------------------
# Include the required packages.
#----------------------------------
use LogInt;         #logging
use Proc;        #proc related calls
use VCSDefines;  #for VCSDef_MonitorExit

#-----------
# globals.
#-----------
use constant CHECK_TYPE_APPLICATION              => 1;
use constant CHECK_TYPE_MAINTAINED_PIDFILE       => 2;
use constant CHECK_TYPE_USE_PROCESSLIST          => 3;


=pod

-----------------------------------------------------------------------------
Function:  Boolean KillProcs( IN ArrayOfStrings ProcNameFilters,
                              IN Boolean LongCommand )

Description:
  Kills all the processes matched by process filters from 
  the long list of the process.

Parameters:
  ProcNameFilters : Filters to match the processes from the process list.
  LongCommand     : Boolean Variable to specify to get the long 
                    list of processes or not.

Return Value:
  Boolean value in the set of {0,1) where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub KillProcs {

  my ( $rlsProcNameFilters,
       $bLongCommand ) = @_;
  
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLError,
                   10415,
                   "VCSHighLevel",
                   "KillProcs", 
                   "Subroutine %s called", $sMyName );
  
  if(!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  #---------------------------------------------------------
  # get the 'ps' data then filter it using our ProcFilters
  #---------------------------------------------------------
  my @lhrProcessDataList = ();

  if($bLongCommand == 0) {
    if(Proc::GetProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10416,
                       "VCSHighLevel",
                       "KillProcs", 
                       "GetProcessListHash returned no proc's" );
      return 0;
    }
  }
  else {
    if(Proc::GetLongProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    }
    else {
      LogInt::Message( LogInt::LLError,
                       10417,
                       "VCSHighLevel",
                       "KillProcs", 
                       "GetLongProcessListHash returned no proc's" );
      return 0;
    }
  }


  my @lhrMatchedProcessDataList = ();

  #------------------------------------------------
  # spin the passed in list of rlsProcNameFilters.
  #------------------------------------------------
  foreach my $sEachFilter (@$rlsProcNameFilters) {
    my @lhrTmpMatchedProcessDataList = ();
    Proc::FilterProcs($sEachFilter, \@lhrProcessDataList, \@lhrTmpMatchedProcessDataList);
    #----------------------------------------
    # add any matched onto the main list.
    #----------------------------------------
    push(@lhrMatchedProcessDataList, @lhrTmpMatchedProcessDataList);
  }

  my @liMatchedProcIds = ();
  my $iMatchedProcCount = Proc::GetValuesForKeyFromProcDataHash("PID", \@lhrMatchedProcessDataList, \@liMatchedProcIds);

  LogInt::Message( LogInt::LLTrace,
                   10418,
                   "VCSHighLevel",
                   "KillProcs", 
                   "iMatchedProcCount is [$iMatchedProcCount]" );

  if($iMatchedProcCount > 0) {
    return Proc::KillPidsFromList(\@liMatchedProcIds, $bLongCommand);
  } 
  else {
    #-----------------
    # no procs found.
    #-----------------
    return 1;
  }
}


=pod

-----------------------------------------------------------------------------
Function: Constant CheckProcs( IN ListOfStrings ProcNameFilters,
                               IN Integer NumberOfEachProcRequired,
                               IN String  ProcType,
                               IN String PidFile,
                               IN String CheckProcType, 
                               IN Boolean LongCommand )

Description:
  Checks the state of the processes.

Parameters:
  ProcNameFilters          : A list of processes strings to match on.
  NumberOfEachProcRequired : The count required for each match.
  ProcType                 : Process Type is one of 
      Proc::PROC_TYPE_PARENT - match only parent processes
      Proc::PROC_TYPE_CHILDREN - match only children processes 
      Proc::PROC_TYPE_ALL - match any (i.e., parent or child) processes
  PidFile                  : Full path of the file to contain matched process 
                             IDs (or application process Ids)
  CheckProcType            : Check Process type is one of the following types:
                            
      VCSHighLevel::CHECK_TYPE_APPLICATION -
        use application maintained pid file
          ProcFilterList should be undefined (or empty) 
          NumberOfEachProcRequired should be undefined or < 2
          ProcType - should be set to PROC_TYPE_ALL
          PidFile - full path name of application pid file

      VCSHighLevel::CHECK_TYPE_MAINTAINED_PIDFILE -
        script will generate/maintain pid file
          ProcFilterList needs to contain at least one entry to match on
          NumberOfEachProcRequired required number of matchs to be found
          ProcType - any of the three, will determine what will be matched on
          PidFile - full path name of pid file to use

      VCSHighLevel::CHECK_TYPE_USE_PROCESSLIST - 
        script will rely on 'ps' command
          ProcFilterList needs to contain at least one entry to match on
          NumberOfEachProcRequired required number of matchs to be found
          ProcType - any of the three, will determine what will be matched on
          PidFile - should be undefined.

  LongCommand              : Boolean to indicate long list of process list.

Return Value:
  Returns one of the monitor exit codes: ONLINE
                                         OFFLINE
-----------------------------------------------------------------------------

=cut

sub CheckProcs {

  my ( $rlsProcNameFilters,
       $iNumberOfEachProcRequired,
       $ProcType, 
       $sPidFile,
       $CheckProcType,
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10419,
                   "VCSHighLevel",
                   "CheckProcs",
                   "Subroutine %s called with: CheckProcType [%s]",
                   $sMyName, $CheckProcType );

  if(!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  #------------------------------------------------------------------------
  # do (some) sanity checking on function inputs
  #
  # calls into Proc can return VCSDef_MonitorExit::PARTIAL_ONLINE
  # VCSDef_MonitorExit::ONLINE and VCSDef_MonitorExit::OFFLINE as well
  # as '0' for error - we convert everything which isnt equal to
  # VCSDef_MonitorExit::ONLINE to be returned as VCSDef_MonitorExit::OFFLINE
  #------------------------------------------------------------------------

  #-------------------------------------------------------------------------------
  # simplest way to sanity check is with a matrix (hash)
  # if a param is required then we'll exit OFFLINE if we dont get it
  # if a param isnt required but we get passed it then we'll log an warning and continue
  # set none required to start (implictly CheckProcType which we check ourselves)
  # if ProcTypeRequired is false then it must be set to Proc::PROC_TYPE_ALL
  # to avoid a warning
  #-------------------------------------------------------------------------------

  my %hParseCheckProcsParams = ( "ProcNameFilterRequired"            => 0,
                                 "NumberOfEachProcRequiredRequired"  => 0,
                                 "ProcTypeRequired"                  => 0,
                                 "PidFileRequired"                   => 0,
                                 );

  if(CHECK_TYPE_APPLICATION == $CheckProcType) {
    $hParseCheckProcsParams{"PidFileRequired"} = 1;
  } 
  elsif(CHECK_TYPE_MAINTAINED_PIDFILE == $CheckProcType) {
    $hParseCheckProcsParams{"ProcNameFilterRequired"}           = 1;
    $hParseCheckProcsParams{"NumberOfEachProcRequiredRequired"} = 1;
    $hParseCheckProcsParams{"ProcTypeRequired"}                 = 1;
    $hParseCheckProcsParams{"PidFileRequired"}                  = 1;
  } 
  elsif(CHECK_TYPE_USE_PROCESSLIST == $CheckProcType) {
    $hParseCheckProcsParams{"ProcNameFilterRequired"}           = 1;
    $hParseCheckProcsParams{"NumberOfEachProcRequiredRequired"} = 1;
    $hParseCheckProcsParams{"ProcTypeRequired"}                 = 1;
  } 
  else {
    #unrecognised $CheckProcType - exit now with offline
    LogInt::Message( LogInt::LLError, 
                     10420,
                     "VCSHighLevel",
                     "CheckProcs",
                     "CheckProcType:[$CheckProcType] is not recognised");
    return VCSDef_MonitorExit::OFFLINE;

  }

  #--------------------------------------
  # Do ProcNameFilterRequired checking.
  #--------------------------------------
  if($hParseCheckProcsParams{"ProcNameFilterRequired"}) {
    #-------------------------------------
    # check we were passed some filters.
    #-------------------------------------
    if(defined @$rlsProcNameFilters) {
      my $iTestProcNameFiltersCount = @$rlsProcNameFilters;
      if($iTestProcNameFiltersCount < 1) {
        #-------------------------------------------
        # nothing to filter on - return offline
        #-------------------------------------------
        LogInt::Message( LogInt::LLError, 
                         10421,
                         "VCSHighLevel",
                         "CheckProcs",
                         "No ProcNameFilters given" );
        return VCSDef_MonitorExit::OFFLINE;
      } 
      else {
        #-----------------------------
        # print trace and continue
        #-----------------------------
        my $sProcNameFiltersString = join(", ", @$rlsProcNameFilters);
        LogInt::Message( LogInt::LLTrace,
                         10422,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcNameFilters:[$sProcNameFiltersString]" );
      }
    } 
    else {
      #----------------------------------------
      # nothing to filter on - return offline
      #----------------------------------------
      LogInt::Message( LogInt::LLError,
                       10423,
                       "VCSHighLevel",
                       "CheckProcs",
                       "No ProcNameFilters given" );
      return VCSDef_MonitorExit::OFFLINE;
    }

  } 
  else {
    #----------------------------------------
    # check we wernt passed some filters.
    #----------------------------------------
    if(defined @$rlsProcNameFilters) {
      my $iTestProcNameFiltersCount = @$rlsProcNameFilters;
      if($iTestProcNameFiltersCount < 1) {
        #---------------------------
        # list is empty continue.
        #---------------------------
      }
      else {
        #--------------------------
        # print warn and continue
        #--------------------------
        my $sProcNameFiltersString = join(", ", @$rlsProcNameFilters);
        LogInt::Message( LogInt::LLWarn,
                         10424,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcNameFilters should be empty; got [$sProcNameFiltersString]");
      }
    } 
    else {
      #list is undefined continue
    }
  }

  #----------------------------------------------
  # Do NumberOfEachProcRequiredRequired checking
  #----------------------------------------------
  if($hParseCheckProcsParams{"NumberOfEachProcRequiredRequired"}) {
    #----------------------------------------------------------
    # they require at least some of each proc to be found
    #----------------------------------------------------------
    if(defined $iNumberOfEachProcRequired) {
      if($iNumberOfEachProcRequired < 1) {
        #-------------------------------------------
        # they required less that 1 - return error
        #-------------------------------------------
        LogInt::Message( LogInt::LLError, 
                         10425,
                         "VCSHighLevel",
                         "CheckProcs",
                         "NumberOfEachProcRequired:[$iNumberOfEachProcRequired] is less than 1");
        return VCSDef_MonitorExit::OFFLINE;
      } 
      else {
        #-------------------------------
        # print trace and continue
        #-------------------------------
        LogInt::Message( LogInt::LLTrace,
                         10426,
                         "VCSHighLevel",
                         "CheckProcs",
                         "iNumberOfEachProcRequired:[$iNumberOfEachProcRequired]" );
      }
    } 
    else {
      #----------------------------------------------
      # iNumberOfEachProcRequired was undefined
      #----------------------------------------------
      LogInt::Message( LogInt::LLError,
                       10427,
                       "VCSHighLevel",
                       "CheckProcs",
                       "NumberOfEachProcRequired was undefined");
      return VCSDef_MonitorExit::OFFLINE;
    }

  } 
  else {
    #-----------------------------------------------
    # they dont require a count of procs returned
    #-----------------------------------------------
    if(defined $iNumberOfEachProcRequired) {
      if($iNumberOfEachProcRequired < 1) {
        #continue
      }
      else {
        #----------------------------------------------
        # they asked for more that 1 - give warning.
        #----------------------------------------------
        LogInt::Message( LogInt::LLWarn,
                         10428,
                         "VCSHighLevel",
                         "CheckProcs",
                         "iNumberOfEachProcRequired:[$iNumberOfEachProcRequired] is more than 1");
      }
    } 
    else { 
      #iNumberOfEachProcRequired was undefined continue
    }
  }

  #-------------------------------------------------------------------------------------- 
  # Do ProcTypeRequired checking
  # if not required it must be set to Proc::PROC_TYPE_ALL to avoid a warning
  #-------------------------------------------------------------------------------------- 
  if($hParseCheckProcsParams{"ProcTypeRequired"}) {
    #----------------------------------------------------------
    # check they specified what type of Process they required
    #----------------------------------------------------------
    if(defined $ProcType) {
      if(Proc::PROC_TYPE_PARENT == $ProcType) {
        #-----------------------------
        # print trace and continue.
        #-----------------------------
        LogInt::Message( LogInt::LLTrace, 
                         10429,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcType:[PROC_TYPE_PARENT]" );
      } 
      elsif(Proc::PROC_TYPE_CHILDREN == $ProcType) {
        #-----------------------------
        # print trace and continue
        #-----------------------------
        LogInt::Message( LogInt::LLTrace,
                         10430,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcType:[PROC_TYPE_CHILDREN]");
      }  
      elsif(Proc::PROC_TYPE_ALL == $ProcType) {
        #-------------------------------
        # print trace and continue
        #-------------------------------
        LogInt::Message( LogInt::LLTrace,
                         10431,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcType:[PROC_TYPE_ALL]" );
      } 
      else {
        #---------------------------------------------
        # proc type not recognised - return offline
        #---------------------------------------------
        LogInt::Message( LogInt::LLError,
                         10432,
                         "VCSHighLevel",
                         "CheckProcs",
                         "ProcType:[$ProcType] is unrecognised (not implemented)");
        return VCSDef_MonitorExit::OFFLINE;
      }
    }
    else {
      #------------------------------------------
      # proc type not defined - return offline
      #------------------------------------------
      LogInt::Message( LogInt::LLError,
                       10433,
                       "VCSHighLevel",
                       "CheckProcs",
                       "ProcType not defined" );
      return VCSDef_MonitorExit::OFFLINE;
    }

  } 
  else {
    if(defined $ProcType) {
       if(Proc::PROC_TYPE_ALL == $ProcType) {
         #continue
       }
       else {
         #-----------------------------------------------------------
         #they didnt set ProcType = Proc::PROC_TYPE_ALL
         #-----------------------------------------------------------
         LogInt::Message( LogInt::LLWarn,
                          10434,
                          "VCSHighLevel",
                          "CheckProcs",
                          "ProcType:[$ProcType] not set to Proc::PROC_TYPE_ALL");
       } 
    }
    else {
      LogInt::Message( LogInt::LLWarn,
                       10435,
                       "VCSHighLevel",
                       "CheckProcs",
                       "ProcType is undefined!");
    }
  }

  #--------------------------
  # Do sPidFile checking
  #--------------------------
  if($hParseCheckProcsParams{"PidFileRequired"}) {
    if(defined $sPidFile) {
      #------------------------------
      # print trace and continue
      #------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10436,
                       "VCSHighLevel",
                       "CheckProcs",
                       "PidFile:[$sPidFile]");
    } 
    else {
      #--------------------------------------
      # no pid file given - return offline
      #--------------------------------------
      LogInt::Message( LogInt::LLError,
                       10437,
                       "VCSHighLevel",
                       "CheckProcs",
                       "no pid file specified" );
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
  else {
    if(defined $sPidFile) {
      #----------------------------------
      # they shouldnt specify a pidfile
      #----------------------------------
      LogInt::Message( LogInt::LLWarn,
                       10438,
                       "VCSHighLevel",
                       "CheckProcs",
                       "PidFile should not be defined");
    } 
    else {
      #no pid file given - continue
    }
  }

  #-------------------------------------------------------------------------
  # if we get here then call the relevant calls into 
  # Proc - based on $CheckProcType
  #
  # calls into Proc can return VCSDef_MonitorExit::PARTIAL_ONLINE
  # VCSDef_MonitorExit::ONLINE and VCSDef_MonitorExit::OFFLINE as well
  # as '0' for error - we convert everything which isnt equal to
  # VCSDef_MonitorExit::ONLINE to be returned as VCSDef_MonitorExit::OFFLINE
  #-------------------------------------------------------------------------
  my $iCheckProcReturnCode;
  if(CHECK_TYPE_APPLICATION == $CheckProcType) {
    $iCheckProcReturnCode = Proc::CheckProcApplicationPidFile($sPidFile, $bLongCommand);
  }
  elsif(CHECK_TYPE_MAINTAINED_PIDFILE == $CheckProcType) {
    $iCheckProcReturnCode = Proc::CheckProcMaintainedPidFile( $rlsProcNameFilters, 
                                                                  $iNumberOfEachProcRequired,
                                                                  $ProcType, 
                                                                  $sPidFile, 
                                                                  $bLongCommand );

  } 
  elsif(CHECK_TYPE_USE_PROCESSLIST == $CheckProcType) {
    $iCheckProcReturnCode = Proc::CheckProcProcessList( $rlsProcNameFilters, 
                                                            $iNumberOfEachProcRequired,
                                                            $ProcType,
                                                            $bLongCommand );

  } 
  else {
    #-----------------------------------------------------------
    # we checked this at the top - but check it here anyway
    # unrecognised $CheckProcType - exit now with offline
    #-----------------------------------------------------------
    LogInt::Message( LogInt::LLError,
                     10439,
                     "VCSHighLevel",
                     "CheckProcs",
                     "CheckProcType:[$CheckProcType] not recognised");
    return VCSDef_MonitorExit::OFFLINE;
  }

  if(VCSDef_MonitorExit::ONLINE == $iCheckProcReturnCode) {
    return VCSDef_MonitorExit::ONLINE; 
  } 
  else {
    return VCSDef_MonitorExit::OFFLINE; 
  }
}


1;

